@@ -15,15 +15,11 @@
 
 #include "adm_locl.h"
 
-RCSID("$Id: kdb_util.c,v 1.28 1996/11/17 20:22:58 assar Exp $");
-
-static char *prog;
+RCSID("$Id: kdb_util.c,v 1.40 1999/07/05 21:43:52 assar Exp $");
 
 static des_cblock master_key, new_master_key;
 static des_key_schedule master_key_schedule, new_master_key_schedule;
 
-#define zaptime(foo) memset((foo), 0, sizeof(*(foo)))
-
 /* cv_key is a procedure which takes a principle and changes its key, 
    either for a new method of encrypting the keys, or a new master key.
    if cv_key is null no transformation of key is done (other than net byte
@@ -40,7 +36,7 @@ print_time(FILE *file, time_t timeval)
     struct tm *tm;
     tm = gmtime(&timeval);
     fprintf(file, " %04d%02d%02d%02d%02d",
-            tm->tm_year < 1900 ? tm->tm_year + 1900: tm->tm_year,
+	    tm->tm_year + 1900,
             tm->tm_mon + 1,
             tm->tm_mday,
             tm->tm_hour,
@@ -54,11 +50,10 @@ time_explode(char *cp)
     struct tm tp;
     int local;
 
-    zaptime(&tp);			/* clear out the struct */
+    memset(&tp, 0, sizeof(tp));	/* clear out the struct */
     
     if (strlen(cp) > 10) {		/* new format */
-	strncpy(wbuf, cp, 4);
-	wbuf[4] = 0;
+	strcpy_truncate(wbuf, cp, sizeof(wbuf));
 	tp.tm_year = atoi(wbuf) - 1900;
 	cp += 4;			/* step over the year */
 	local = 0;			/* GMT */
@@ -88,13 +83,13 @@ time_explode(char *cp)
     wbuf[1] = *cp++;
     tp.tm_min = atoi(wbuf);
 
-
     return(tm2time(tp, local));
 }
 
 static int
-dump_db_1(void *arg, Principal *principal)
-{	    /* replace null strings with "*" */
+dump_db_1(void *arg,
+	  Principal *principal)	/* replace null strings with "*" */
+{
     struct callback_args *a = (struct callback_args *)arg;
     
     if (principal->instance[0] == '\0') {
@@ -137,7 +132,7 @@ dump_db (char *db_file, FILE *output_file, void (*cv_key) (Principal *))
     a.cv_key = cv_key;
     a.output_file = output_file;
     
-    kerb_db_iterate ((k_iter_proc_t)dump_db_1, &a);
+    kerb_db_iterate (dump_db_1, &a);
     return fflush(output_file);
 }
 
@@ -158,10 +153,8 @@ add_file(void *db, FILE *file)
 	memset(&pr, 0, sizeof(pr));
 	errno = 0;
 	if(fgets(line, sizeof(line), file) == NULL){
-	    if(errno != 0){
-		perror("fgets");
-		return 1;
-	    }
+	    if(errno != 0)
+	      err (1, "fgets");
 	    break;
 	}
 	lineno++;
@@ -173,7 +166,7 @@ add_file(void *db, FILE *file)
 		     exp_date, mod_date,
 		     pr.mod_name, pr.mod_instance);
 	if(ret != 12){
-	    fprintf(stderr, "Line %d malformed (ignored)\n", lineno);
+	    warnx("Line %d malformed (ignored)", lineno);
 	    continue;
 	}
 	pr.key_low = ntohl (key[0]);
@@ -190,8 +183,8 @@ add_file(void *db, FILE *file)
 	if (pr.mod_instance[0] == '*')
 	    pr.mod_instance[0] = 0;
 	if (kerb_db_update(db, &pr, 1) != 1) {
-	    fprintf(stderr, "Couldn't store %s.%s: %s; load aborted\n",
-		    pr.name, pr.instance, strerror(errno));
+	    warn ("store %s.%s aborted",
+		  pr.name, pr.instance);
 	    return 1;
 	}
     }
@@ -202,37 +195,27 @@ static void
 load_db (char *db_file, FILE *input_file)
 {
     long *db;
-    int     temp1;
     int code;
     char *temp_db_file;
 
-    temp1 = strlen(db_file)+2;
-    temp_db_file = malloc (temp1);
-    strcpy(temp_db_file, db_file);
-    strcat(temp_db_file, "~");
+    asprintf (&temp_db_file, "%s~", db_file);
+    if(temp_db_file == NULL)
+	errx (1, "out of memory");
 
     /* Create the database */
-    if ((code = kerb_db_create(temp_db_file)) != 0) {
-	fprintf(stderr, "Couldn't create temp database %s: %s\n",
-		temp_db_file, strerror(code));
-	exit(1);
-    }
+    if ((code = kerb_db_create(temp_db_file)) != 0)
+	err (1, "creating temp database %s", temp_db_file);
     kerb_db_set_name(temp_db_file);
     db = kerb_db_begin_update();
-    if (db == NULL) {
-	fprintf(stderr, "Couldn't open temp database %s; %s\n",
-		temp_db_file, strerror(code));
-	exit(1);
-    }
+    if (db == NULL)
+	err (1, "opening temp database %s", temp_db_file);
     
-    if(add_file(db, input_file)){
-	fprintf(stderr, "Load aborted.\n");
-	exit(1);
-    }
+    if(add_file(db, input_file))
+	errx (1, "Load aborted");
 
     kerb_db_end_update(db);
     if ((code = kerb_db_rename(temp_db_file, db_file)) != 0)
-	perror("database rename failed");
+        warn("database rename failed");
     fclose(input_file);
     free(temp_db_file);
 }
@@ -243,13 +226,10 @@ merge_db(char *db_file, FILE *input_file)
     void *db;
     
     db = kerb_db_begin_update();
-    if(db == NULL){
-	fprintf(stderr, "Couldn't open database: %s\n", strerror(errno));
-	exit(1);
-    }
-    if(add_file(db, input_file)){
-	fprintf(stderr, "Merge aborted!\n");
-    }
+    if(db == NULL)
+        err (1, "Couldn't open database");
+    if(add_file(db, input_file))
+        errx (1, "Merge aborted");
     kerb_db_end_update(db);
 }
 
@@ -259,23 +239,20 @@ update_ok_file (char *file_name)
     /* handle slave locking/failure stuff */
     char *file_ok;
     int fd;
-    static char ok[]=".dump_ok";
 
-    if ((file_ok = (char *)malloc(strlen(file_name) + strlen(ok) + 1))
-	== NULL) {
-	fprintf(stderr, "kdb_util: out of memory.\n");
-	perror ("malloc");
-	exit (1);
-    }
-    strcpy(file_ok, file_name);
-    strcat(file_ok, ok);
-    if ((fd = open(file_ok, O_WRONLY|O_CREAT|O_TRUNC, 0400)) < 0) {
-	fprintf(stderr, "Error creating 'ok' file, '%s'", file_ok);
-	perror("");
-	exit (1);
-    }
+    asprintf (&file_ok, "%s.dump_ok", file_name);
+    if (file_ok == NULL)
+      errx (1, "out of memory");
+    if ((fd = open(file_ok, O_WRONLY|O_CREAT|O_TRUNC, 0600)) < 0)
+        err (1, "Error creating %s", file_ok);
     free(file_ok);
     close(fd);
+    /*
+     * Some versions of BSD don't update the mtime in the above open so
+     * we call utimes just in case.
+     */
+    if (utime(file_name, NULL) < 0)
+	err (1, "utime %s", file_name);
 }
 
 static void
@@ -294,10 +271,12 @@ convert_key_new_master (Principal *p)
     (p->key_version)++;
   } else {
     copy_to_key(&p->key_low, &p->key_high, key);
-    kdb_encrypt_key (&key, &key, &master_key, master_key_schedule, DES_DECRYPT);
+    kdb_encrypt_key (&key, &key, &master_key,
+		     master_key_schedule, DES_DECRYPT);
   }
 
-  kdb_encrypt_key (&key, &key, &new_master_key, new_master_key_schedule, DES_ENCRYPT);
+  kdb_encrypt_key (&key, &key, &new_master_key,
+		   new_master_key_schedule, DES_ENCRYPT);
 
   copy_from_key(key, &(p->key_low), &(p->key_high));
   memset(key, 0, sizeof (key));  /* a little paranoia ... */
@@ -318,16 +297,14 @@ static void
 convert_new_master_key (char *db_file, FILE *out)
 {
 #ifdef RANDOM_MKEY
-  fprintf(stderr, "Sorry, this function is not available with "
-	  "the new master key scheme.\n");
-  exit(1);
+  errx (1, "Sorry, this function is not available with "
+	"the new master key scheme.");
 #else
   printf ("\n\nEnter the CURRENT master key.");
   if (kdb_get_master_key (KDB_GET_PROMPT, &master_key,
 			  master_key_schedule) != 0) {
-    fprintf (stderr, "%s: Couldn't get master key.\n", prog);
     clear_secrets ();
-    exit (1);
+    errx (1, "Couldn't get master key.");
   }
 
   if (kdb_verify_master_key (&master_key, master_key_schedule, stderr) < 0) {
@@ -338,16 +315,21 @@ convert_new_master_key (char *db_file, FILE *out)
   printf ("\n\nNow enter the NEW master key.  Do not forget it!!");
   if (kdb_get_master_key (KDB_GET_TWICE, &new_master_key,
 			  new_master_key_schedule) != 0) {
-    fprintf (stderr, "%s: Couldn't get new master key.\n", prog);
     clear_secrets ();
-    exit (1);
+    errx (1, "Couldn't get new master key.");
   }
 
   dump_db (db_file, out, convert_key_new_master);
   {
-    char fname[128];
-    sprintf(fname, "%s.new", MKEYFILE);
+    char *fname;
+
+    asprintf(&fname, "%s.new", MKEYFILE);
+    if(fname == NULL) {
+      clear_secrets();
+      errx(1, "malloc: failed");
+    }
     kdb_kstash(&new_master_key, fname);
+    free(fname);
   }
 #endif /* RANDOM_MKEY */
 }
@@ -384,19 +366,16 @@ convert_old_format_db (char *db_file, FILE *out)
 
   if (kdb_get_master_key (KDB_GET_PROMPT, &master_key,
 			  master_key_schedule) != 0L) {
-    fprintf (stderr, "%s: Couldn't get master key.\n", prog);
     clear_secrets();
-    exit (1);
+    errx (1, "Couldn't get master key.");
   }
 
   /* can't call kdb_verify_master_key because this is an old style db */
   /* lookup the master key version */
   n = kerb_get_principal(KERB_M_NAME, KERB_M_INST, principal_data,
 			 1 /* only one please */, &more);
-  if ((n != 1) || more) {
-    fprintf(stderr, "verify_master_key: Kerberos error on master key lookup, %d found.\n", n);
-    exit (1);
-  }
+  if ((n != 1) || more)
+    errx (1, "verify_master_key: Kerberos error on master key lookup, %d found.\n", n);
 
   /* set up the master key */
   fprintf(stderr, "Current Kerberos master key version is %d.\n",
@@ -444,12 +423,15 @@ main(int argc, char **argv)
     }       op;
     char *file_name;
     char *db_name;
-    prog = argv[0];
+
+    set_progname (argv[0]);
     
     if (argc != 3 && argc != 4) {
 	fprintf(stderr, "Usage: %s operation file [database name].\n",
 		argv[0]);
-	fprintf(stderr, "Operation is one of: load, merge, dump, slave_dump, new_master_key\n");
+	fprintf(stderr, "Operation is one of: "
+		"load, merge, dump, slave_dump, new_master_key, "
+		"convert_old_db\n");
 	exit(1);
     }
     if (argc == 3)
@@ -463,10 +445,8 @@ main(int argc, char **argv)
     if(ret && strcmp(argv[1], "load") == 0)
        ret = kerb_db_create (db_name);
 
-    if(ret){
-	perror("Can't open database");
-	return 1;
-    }
+    if(ret)
+      err (1, "Can't open database");
 
     if (!strcmp(argv[1], "load"))
 	op = OP_LOAD;
@@ -481,39 +461,28 @@ main(int argc, char **argv)
     else if (!strcmp(argv[1], "convert_old_db"))
         op = OP_CONVERT_OLD_DB;
     else {
-	fprintf(stderr,
-	    "%s: %s is an invalid operation.\n", prog, argv[1]);
-	fprintf(stderr,
-	    "%s: Valid operations are \"dump\", \"slave_dump\",", argv[0]);
-	fprintf(stderr,
-		"\"load\", \"new_master_key\", and \"convert_old_db\".\n");
+        warnx ("%s is an invalid operation.", argv[1]);
+	warnx ("Valid operations are \"load\", \"merge\", "
+	       "\"dump\", \"slave_dump\", \"new_master_key\", "
+	       "and \"convert_old_db\"");
 	return 1;
     }
 
     file_name = argv[2];
     file = fopen(file_name, (op == OP_LOAD || op == OP_MERGE) ? "r" : "w");
-    if (file == NULL) {
-	fprintf(stderr, "%s: Unable to open %s\n", prog, argv[2]);
-	perror("open");
-	return 1;
-    }
+    if (file == NULL)
+        err (1, "open %s", argv[2]);
 
     switch (op) {
     case OP_DUMP:
       if ((dump_db (db_name, file, (void (*)(Principal *)) 0) == EOF) ||
-	  (fclose(file) == EOF)) {
-	  fprintf(stderr, "error on file %s:", file_name);
-	  perror("");
-	  return 1;
-      }
+	  (fclose(file) == EOF))
+	  err (1, "%s", file_name);
       break;
     case OP_SLAVE_DUMP:
       if ((dump_db (db_name, file, (void (*)(Principal *)) 0) == EOF) ||
-	  (fclose(file) == EOF)) {
-	  fprintf(stderr, "error on file %s:", file_name);
-	  perror("");
-	  return 1;
-      }
+	  (fclose(file) == EOF))
+	err (1, "%s", file_name);
       update_ok_file (file_name);
       break;
     case OP_LOAD:
