@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -36,7 +36,7 @@
  * SUCH DAMAGE.
  */
 
-/* $Id: bsd_locl.h,v 1.102 1998/04/04 13:11:51 assar Exp $ */
+/* $Id: bsd_locl.h,v 1.109.2.1 1999/07/22 03:13:49 assar Exp $ */
 
 #define LOGALL
 #define KERBEROS
@@ -49,7 +49,7 @@
 #endif
 
 /* Any better way to test NO_MOTD? */
-#if (SunOS == 5) || defined(__hpux)
+#if (SunOS >= 50) || defined(__hpux)
 #define NO_MOTD
 #endif
 
@@ -62,6 +62,7 @@
 #include <stdlib.h>
 #include <ctype.h>
 #include <setjmp.h>
+#include <limits.h>
 
 #include <stdarg.h>
 
@@ -72,6 +73,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_LIBUTIL_H
+#include <libutil.h>
+#endif
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
 #endif
@@ -148,7 +152,7 @@
 #include <netdb.h>
 #endif
 
-#if defined(HAVE_SYS_IOCTL_H) && SunOS != 4
+#if defined(HAVE_SYS_IOCTL_H) && SunOS != 40
 #include <sys/ioctl.h>
 #endif
 #ifdef HAVE_SYS_IOCCOM_H
@@ -245,9 +249,14 @@
 #endif /* HAVE_USERPW_H */
 
 #ifdef HAVE_USERSEC_H
+struct aud_rec;
 #include <usersec.h>
 #endif /* HAVE_USERSEC_H */
 
+#ifdef HAVE_OSFC2
+#include "/usr/include/prot.h"
+#endif
+
 #ifndef PRIO_PROCESS
 #define PRIO_PROCESS 0
 #endif
@@ -258,6 +267,9 @@
 
 #ifdef SOCKS
 #include <socks.h>
+/* This doesn't belong here. */
+struct tm *localtime(const time_t *);
+struct hostent  *gethostbyname(const char *);
 #endif
 
 #include <des.h>
@@ -292,6 +304,9 @@ int susystem(char *s, int userid);
 int forkpty(int *amaster, char *name,
 	    struct termios *termp, struct winsize *winp);
 
+int forkpty_truncate(int *amaster, char *name, size_t name_sz,
+		     struct termios *termp, struct winsize *winp);
+
 #ifndef MODEMASK
 #define	MODEMASK	(S_ISUID|S_ISGID|S_ISTXT|S_IRWXU|S_IRWXG|S_IRWXO)
 #endif
@@ -313,7 +328,7 @@ extern char **environ;
 void sysv_newenv(int argc, char **argv, struct passwd *pwd,
 		 char *term, int pflag);
 
-int login_access(char *user, char *from);
+int login_access(struct passwd *user, char *from);
 void fatal(int f, const char *msg, int syserr);
 
 extern int LEFT_JUSTIFIED;
@@ -322,6 +337,10 @@ int des_enc_read(int fd,char *buf,int len,des_key_schedule sched,
 int des_enc_write(int fd,char *buf,int len,des_key_schedule sched,
 	des_cblock *iv);
 
+/* used in des_read and des_write */
+#define DES_RW_MAXWRITE	(1024*16)
+#define DES_RW_BSIZE	(DES_RW_MAXWRITE+4)
+
 void sysv_defaults(void);
 void utmp_login(char *tty, char *username, char *hostname);
 void sleepexit (int);
@@ -341,7 +360,7 @@ void sleepexit (int);
 #ifndef _POSIX_VDISABLE
 #define _POSIX_VDISABLE 0
 #endif /* _POSIX_VDISABLE */
-#if SunOS == 4
+#if SunOS == 40
 #include <sys/ttold.h>
 #endif
 
@@ -379,3 +398,5 @@ char *make_id (char *tty);
 void prepare_utmp (struct utmp *utmp, char *tty, char *username,
 		   char *hostname);
 #endif
+
+int do_osfc2_magic(uid_t);
