@@ -38,8 +38,14 @@
  */
 
 #include "bsd_locl.h"
+#ifdef HAVE_CAPABILITY_H
+#include <capability.h>
+#endif
+#ifdef HAVE_SYS_CAPABILITY_H
+#include <sys/capability.h>
+#endif
 
-RCSID("$Id: login.c,v 1.112 1998/05/18 19:28:31 joda Exp $");
+RCSID("$Id: login.c,v 1.120.2.1 1999/08/19 13:38:31 assar Exp $");
 
 #ifdef OTP
 #include <otp.h>
@@ -244,7 +250,9 @@ main(int argc, char **argv)
 			if (uid)
 				errx(1, "-h option: %s", strerror(EPERM));
 			hflag = 1;
-			strncpy(full_hostname, optarg, sizeof(full_hostname)-1);
+			strcpy_truncate(full_hostname,
+					optarg,
+					sizeof(full_hostname));
 			if (domain && (p = strchr(optarg, '.')) &&
 			    strcasecmp(p, domain) == 0)
 				*p = 0;
@@ -267,7 +275,9 @@ main(int argc, char **argv)
 				exit(1);
                         }
 			rflag = 1;
-			strncpy(full_hostname, optarg, sizeof(full_hostname)-1);
+			strcpy_truncate(full_hostname,
+					optarg,
+					sizeof(full_hostname));
 			if (domain && (p = strchr(optarg, '.')) &&
 			    strcasecmp(p, domain) == 0)
 				*p = 0;
@@ -373,7 +383,7 @@ main(int argc, char **argv)
 				badlogin(tbuf);
 			failures = 0;
 		}
-		strcpy(tbuf, username);
+		strcpy_truncate(tbuf, username, sizeof(tbuf));
 
 		pwd = paranoid_getpwnam (username);
 
@@ -524,26 +534,45 @@ main(int argc, char **argv)
 	    struct udb *udb;
 	    long t;
 	    const long maxcpu = 46116860184; /* some random constant */
+	    
+	    if(setjob(pwd->pw_uid, 0) < 0) 
+		warn("setjob");
+
 	    udb = getudbnam(pwd->pw_name);
 	    if(udb == UDB_NULL)
 		errx(1, "Failed to get UDB entry.");
+
+	    /* per process cpu limit */
 	    t = udb->ue_pcpulim[UDBRC_INTER];
 	    if(t == 0 || t > maxcpu)
 		t = CPUUNLIM;
 	    else
-		t *= 100 * CLOCKS_PER_SEC;
+		t *= CLK_TCK;
 
 	    if(limit(C_PROC, 0, L_CPU, t) < 0)
-		warn("limit C_PROC");
+		warn("limit process cpu");
 
+	    /* per process memory limit */
+	    if(limit(C_PROC, 0, L_MEM, udb->ue_pmemlim[UDBRC_INTER]) < 0)
+		warn("limit process memory");
+
+	    /* per job cpu limit */
 	    t = udb->ue_jcpulim[UDBRC_INTER];
 	    if(t == 0 || t > maxcpu)
 		t = CPUUNLIM;
 	    else
-		t *= 100 * CLOCKS_PER_SEC;
+		t *= CLK_TCK;
 
-	    if(limit(C_JOBPROCS, 0, L_CPU, t) < 0)
-		warn("limit C_JOBPROCS");
+	    if(limit(C_JOB, 0, L_CPU, t) < 0)
+		warn("limit job cpu");
+	    
+	    /* per job processor limit */
+	    if(limit(C_JOB, 0, L_CPROC, udb->ue_jproclim[UDBRC_INTER]) < 0)
+		warn("limit job processors");
+
+	    /* per job memory limit */
+	    if(limit(C_JOB, 0, L_MEM, udb->ue_jmemlim[UDBRC_INTER]) < 0)
+		warn("limit job memory");
 
 	    nice(udb->ue_nice[UDBRC_INTER]);
 	}
@@ -605,9 +634,11 @@ main(int argc, char **argv)
 	 */
 	login_fbtab(tty, pwd->pw_uid, pwd->pw_gid);
 
-	chown(ttyn, pwd->pw_uid,
-	    (gr = getgrnam(TTYGRPNAME)) ? gr->gr_gid : pwd->pw_gid);
-	chmod(ttyn, S_IRUSR | S_IWUSR | S_IWGRP);
+	if (chown(ttyn, pwd->pw_uid,
+	    (gr = getgrnam(TTYGRPNAME)) ? gr->gr_gid : pwd->pw_gid) < 0)
+	  err(1, "chown tty failed");
+	if (chmod(ttyn, S_IRUSR | S_IWUSR | S_IWGRP) < 0)
+	  err(1, "chmod tty failed");
 	setgid(pwd->pw_gid);
 
 	initgroups(username, pwd->pw_gid);
@@ -623,7 +654,7 @@ main(int argc, char **argv)
          * that LD_* and IFS are never preserved.
          */
 	if (term[0] == '\0')
-		strncpy(term, stypeof(tty), sizeof(term));
+		strcpy_truncate(term, stypeof(tty), sizeof(term));
         /* set up a somewhat censored environment. */
         sysv_newenv(argc, argv, pwd, term, pflag);
 #ifdef KERBEROS
@@ -635,12 +666,13 @@ main(int argc, char **argv)
 		syslog(LOG_INFO, "DIALUP %s, %s", tty, pwd->pw_name);
 
 	/* If fflag is on, assume caller/authenticator has logged root login. */
-	if (rootlogin && fflag == 0)
+	if (rootlogin && fflag == 0) {
 		if (hostname)
 			syslog(LOG_NOTICE, "ROOT LOGIN (%s) ON %s FROM %s",
 			    username, tty, hostname);
 		else
 			syslog(LOG_NOTICE, "ROOT LOGIN (%s) ON %s", username, tty);
+	}
 
 #ifdef KERBEROS
 	if (!quietlog && notickets == 1 && !noticketsdontcomplain)
@@ -683,7 +715,7 @@ main(int argc, char **argv)
 #endif /* NO_MOTD */
 
 #ifdef LOGIN_ACCESS
-	if (login_access(pwd->pw_name, hostname ? full_hostname : tty) == 0) {
+	if (login_access(pwd, hostname ? full_hostname : tty) == 0) {
 		printf("Permission denied\n");
 		if (hostname)
 			syslog(LOG_NOTICE, "%s LOGIN REFUSED FROM %s",
@@ -702,9 +734,8 @@ main(int argc, char **argv)
 	signal(SIGTSTP, SIG_IGN);
 #endif
 
-	tbuf[0] = '-';
-	strcpy(tbuf + 1, (p = strrchr(pwd->pw_shell, '/')) ?
-	       p + 1 : pwd->pw_shell);
+	p = strrchr(pwd->pw_shell, '/');
+	snprintf (tbuf, sizeof(tbuf), "-%s", p ? p + 1 : pwd->pw_shell);
 
 #ifdef HAVE_SETLOGIN
      	if (setlogin(pwd->pw_name) < 0)
@@ -719,6 +750,26 @@ main(int argc, char **argv)
 #if defined(SYSV_SHADOW) && defined(HAVE_GETSPNAM)
 	spwd = getspnam (username);
 	endspent ();
+#endif
+	/* perhaps work some magic */
+	if(do_osfc2_magic(pwd->pw_uid))
+	    sleepexit(1);
+#ifdef HAVE_SGI_GETCAPABILITYBYNAME
+	/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX */
+	{
+	    struct user_cap *ucap = sgi_getcapabilitybyname(pwd->pw_name);
+	    cap_t cap;
+	    if(ucap == NULL)
+		cap = cap_from_text("all=");
+	    else
+		cap = cap_from_text(ucap->ca_default);
+	    if(cap == NULL)
+		err(1, "cap_from_text");
+	    if(cap_set_proc(cap) < 0)
+		err(1, "cap_set_proc");
+	    cap_free(cap);
+	    free(ucap);
+	}
 #endif
 	/* Discard permissions last so can't get killed and drop core. */
 	{
@@ -793,34 +844,36 @@ main(int argc, char **argv)
 static void
 getloginname(int prompt)
 {
-	int ch;
-	char *p;
-	static char nbuf[NBUFSIZ];
+    int ch;
+    char *p;
+    static char nbuf[NBUFSIZ];
 
-	for (;;) {
-                if (prompt)
-                    if (ttyprompt && *ttyprompt)
-                        printf("%s", ttyprompt);
-                    else
-		        printf("login: ");
-		prompt = 1;
-		for (p = nbuf; (ch = getchar()) != '\n'; ) {
-			if (ch == EOF) {
-				badlogin(username);
-				exit(0);
-			}
-			if (p < nbuf + (NBUFSIZ - 1))
-				*p++ = ch;
-		}
-		if (p > nbuf)
-			if (nbuf[0] == '-')
-				warnx("login names may not start with '-'.");
-			else {
-				*p = '\0';
-				username = nbuf;
-				break;
-			}
+    for (;;) {
+	if (prompt) {
+	    if (ttyprompt && *ttyprompt)
+		printf("%s", ttyprompt);
+	    else
+		printf("login: ");
 	}
+	prompt = 1;
+	for (p = nbuf; (ch = getchar()) != '\n'; ) {
+	    if (ch == EOF) {
+		badlogin(username);
+		exit(0);
+	    }
+	    if (p < nbuf + (NBUFSIZ - 1))
+		*p++ = ch;
+	}
+	if (p > nbuf) {
+	    if (nbuf[0] == '-')
+		warnx("login names may not start with '-'.");
+	    else {
+		*p = '\0';
+		username = nbuf;
+		break;
+	    }
+	}
+    }
 }
 
 static int
