@@ -33,11 +33,9 @@
 
 #include "bsd_locl.h"
 
-RCSID("$Id: rcp.c,v 1.31 1996/11/19 04:11:17 assar Exp $");
+RCSID("$Id: rcp.c,v 1.49 1999/07/06 03:17:58 assar Exp $");
 
 /* Globals */
-char *__progname;
-
 static char	dst_realm_buf[REALM_SZ];
 static char	*dest_realm = NULL;
 static int	use_kerberos = 1;
@@ -51,6 +49,9 @@ static u_short	port;
 static uid_t	userid;
 static int pflag, iamremote, iamrecursive, targetshouldbedirectory;
 
+static int argc_copy;
+static char **argv_copy;
+
 #define	CMDNEEDS	64
 static char cmd[CMDNEEDS];		/* must hold "rcp -r -p -d\0" */
 
@@ -60,7 +61,7 @@ void rsource(char *name, struct stat *statp);
 
 CREDENTIALS cred;
 MSG_DAT msg_data;
-struct sockaddr_in foreign, local;
+struct sockaddr_in foreign_addr, local_addr;
 Key_schedule schedule;
 
 KTEXT_ST ticket;
@@ -73,28 +74,19 @@ send_auth(char *h, char *r)
     long opts;
 
     lslen = sizeof(struct sockaddr_in);
-    if (getsockname(rem, (struct sockaddr *)&local, &lslen) < 0)
-	{
-	    perror("getsockname");
-	    exit(1);
-	}
+    if (getsockname(rem, (struct sockaddr *)&local_addr, &lslen) < 0)
+	err(1, "getsockname");
     fslen = sizeof(struct sockaddr_in);
-    if (getpeername(rem, (struct sockaddr *)&foreign, &fslen) < 0)
-	{
-	    perror("getpeername");
-	    exit(1);
-	}
+    if (getpeername(rem, (struct sockaddr *)&foreign_addr, &fslen) < 0)
+	err(1, "getpeername");
     if ((r == NULL) || (*r == '\0'))
 	r = krb_realmofhost(h);
     opts = KOPT_DO_MUTUAL;
     if ((status = krb_sendauth(opts, rem, &ticket, SERVICE_NAME, h, r, 
-			    (unsigned long)getpid(), &msg_data, &cred, schedule, &local, 
-			    &foreign, "KCMDV0.1")) != KSUCCESS)
-	{
-	    fprintf(stderr, "krb_sendauth failure: %s\n", 
-		    krb_get_err_text(status));
-	    exit(1);
-	}
+			       (unsigned long)getpid(), &msg_data, &cred, 
+			       schedule, &local_addr, 
+			       &foreign_addr, "KCMDV0.1")) != KSUCCESS)
+	errx(1, "krb_sendauth failure: %s", krb_get_err_text(status));
 }
 
 static void
@@ -105,26 +97,17 @@ answer_auth(void)
     char inst[INST_SZ], v[9];
 
     lslen = sizeof(struct sockaddr_in);
-    if (getsockname(rem, (struct sockaddr *)&local, &lslen) < 0)
-	{
-	    perror("getsockname");
-	    exit(1);
-	}
+    if (getsockname(rem, (struct sockaddr *)&local_addr, &lslen) < 0)
+	err(1, "getsockname");
     fslen = sizeof(struct sockaddr_in);
-    if(getpeername(rem, (struct sockaddr *)&foreign, &fslen) < 0)
-	{
-	    perror("getperrname");
-	    exit(1);
-	}
-    k_getsockinst(rem, inst);
+    if(getpeername(rem, (struct sockaddr *)&foreign_addr, &fslen) < 0)
+	    err(1, "getperrname");
+    k_getsockinst(rem, inst, sizeof(inst));
     opts = KOPT_DO_MUTUAL;
-    if ((status = krb_recvauth(opts, rem, &ticket, SERVICE_NAME, inst, &foreign, &local, 
+    if ((status = krb_recvauth(opts, rem, &ticket, SERVICE_NAME, inst,
+			       &foreign_addr, &local_addr, 
 			       &kdata, "", schedule, v)) != KSUCCESS)
-	{
-	    fprintf(stderr, "krb_recvauth failure: %s\n", 
-		    krb_get_err_text(status));
-	    exit(1);
-	}
+	errx(1, "krb_recvauth failure: %s", krb_get_err_text(status));
 }
 
 static int
@@ -147,20 +130,31 @@ des_write(int fd, char *buf, int len)
 	return(write(fd, buf, len));
 }
 
-static void
-run_err(const char *fmt, const char *arg1, const char *arg2)
-{
-        char errbuf[1024];
+static void run_err(const char *fmt, ...)
+#ifdef __GNUC__
+__attribute__ ((format (printf, 1, 2)))
+#endif
+;
 
+
+static void
+run_err(const char *fmt, ...)
+{
+	char errbuf[1024];
+
+	va_list args;
+	va_start(args, fmt);
 	++errs;
 #define RCPERR "\001rcp: "
-	strcpy(errbuf, RCPERR);
-	sprintf(errbuf+strlen(RCPERR), fmt, arg1, arg2);
-	strcat(errbuf, "\n");
-	des_write(rem, errbuf, strlen(errbuf));
-
+	strcpy_truncate (errbuf, RCPERR, sizeof(errbuf));
+	vsnprintf (errbuf + strlen(errbuf),
+		   sizeof(errbuf) - strlen(errbuf),
+		   fmt, args);
+	strcat_truncate (errbuf, "\n", sizeof(errbuf));
+	des_write (rem, errbuf, strlen(errbuf));
 	if (!iamremote)
-		warnx(fmt, arg1, arg2);
+		vwarnx(fmt, args);
+	va_end(args);
 }
 
 static void
@@ -186,7 +180,7 @@ allocbuf(BUF *bp, int fd, int blksize)
 	size_t size;
 
 	if (fstat(fd, &stb) < 0) {
-		run_err("fstat: %s", strerror(errno), "");
+		run_err("fstat: %s", strerror(errno));
 		return (0);
 	}
 #ifdef HAVE_ST_BLKSIZE
@@ -204,7 +198,7 @@ allocbuf(BUF *bp, int fd, int blksize)
 		bp->buf = realloc(bp->buf, size);
 	if (bp->buf == NULL) {
 		bp->cnt = 0;
-		run_err("%s", strerror(errno), "");
+		run_err("%s", strerror(errno));
 		return (0);
 	}
 	bp->cnt = size;
@@ -221,16 +215,13 @@ usage(void)
 }
 
 static void
-oldw(const char *fmt, const char *arg)
+oldw(const char *s)
 {
     char *rstar_no_warn = getenv("RSTAR_NO_WARN");
     if (rstar_no_warn == 0)
 	rstar_no_warn = "";
-    if (strncmp(rstar_no_warn, "yes", 3) != 0) {
-	fprintf(stderr, "rcp: ");
-	fprintf(stderr, fmt, arg);
-	fprintf(stderr, ", using standard rcp\n");
-    }
+    if (strncmp(rstar_no_warn, "yes", 3) != 0)
+	warnx("%s, using standard rcp", s);
 }
 
 static RETSIGTYPE
@@ -302,7 +293,7 @@ syserr:			run_err("%s: %s", name, strerror(errno));
 			}
 			/* FALLTHROUGH */
 		default:
-			run_err("%s: not a regular file", name, "");
+			run_err("%s: not a regular file", name);
 			goto next;
 		}
 		if ((last = strrchr(name, '/')) == NULL)
@@ -314,13 +305,13 @@ syserr:			run_err("%s: %s", name, strerror(errno));
 			 * Make it compatible with possible future
 			 * versions expecting microseconds.
 			 */
-			sprintf(buf, "T%ld 0 %ld 0\n",
+			snprintf(buf, sizeof(buf), "T%ld 0 %ld 0\n",
 			    (long)stb.st_mtime, (long)stb.st_atime);
 			des_write(rem, buf, strlen(buf));
 			if (response() < 0)
 				goto next;
 		}
-		sprintf(buf, "C%04o %ld %s\n",
+		snprintf(buf, sizeof(buf), "C%04o %ld %s\n",
 		    (int)stb.st_mode & MODEMASK, (long) stb.st_size, last);
 		des_write(rem, buf, strlen(buf));
 		if (response() < 0)
@@ -364,18 +355,22 @@ rsource(char *name, struct stat *statp)
 	DIR *dirp;
 	struct dirent *dp;
 	char *last, *vect[1], path[MaxPathLen];
+	char *p;
 
 	if (!(dirp = opendir(name))) {
 		run_err("%s: %s", name, strerror(errno));
 		return;
 	}
+	for (p = name + strlen(name) - 1; p >= name && *p == '/'; --p)
+	    *p = '\0';
+
 	last = strrchr(name, '/');
 	if (last == 0)
 		last = name;
 	else
 		last++;
 	if (pflag) {
-		sprintf(path, "T%ld 0 %ld 0\n",
+		snprintf(path, sizeof(path), "T%ld 0 %ld 0\n",
 		    (long)statp->st_mtime, (long)statp->st_atime);
 		des_write(rem, path, strlen(path));
 		if (response() < 0) {
@@ -383,8 +378,8 @@ rsource(char *name, struct stat *statp)
 			return;
 		}
 	}
-	sprintf(path,
-	    "D%04o %d %s\n", (int)statp->st_mode & MODEMASK, 0, last);
+	snprintf(path, sizeof(path),
+		 "D%04o %d %s\n", (int)statp->st_mode & MODEMASK, 0, last);
 	des_write(rem, path, strlen(path));
 	if (response() < 0) {
 		closedir(dirp);
@@ -399,7 +394,11 @@ rsource(char *name, struct stat *statp)
 			run_err("%s/%s: name too long", name, dp->d_name);
 			continue;
 		}
-		sprintf(path, "%s/%s", name, dp->d_name);
+		if (snprintf(path, sizeof(path),
+			     "%s/%s", name, dp->d_name) >= sizeof(path)) {
+			run_err("%s/%s: name too long", name, dp->d_name);
+			continue;
+		}
 		vect[0] = path;
 		source(1, vect);
 	}
@@ -412,8 +411,9 @@ static int
 kerberos(char **host, char *bp, char *locuser, char *user)
 {
         int sock = -1, err;
-again:
+
 	if (use_kerberos) {
+	        setuid(getuid());
 		rem = KSUCCESS;
 		errno = 0;
 		if (dest_realm == NULL)
@@ -440,29 +440,27 @@ again:
 		    (struct sockaddr_in *) NULL, /* foreign addr not used */
 		    0L);	/* authopts */
 		if (err > KSUCCESS && err < MAX_KRB_ERRORS) {
-			fprintf(stderr, "kcmd: %s\n", krb_get_err_text(err));
-			rem = -1;
+		    warnx("kcmd: %s", krb_get_err_text(err));
+		    rem = -1;
 		} else if (err < 0)
 		    rem = -1;
 		else
 		    rem = sock;
 #endif
 		if (rem < 0) {
-			use_kerberos = 0;
-			port=get_shell_port(use_kerberos, doencrypt);
 			if (errno == ECONNREFUSED)
-			    oldw("remote host doesn't support Kerberos", "");
+			    oldw("remote host doesn't support Kerberos");
 			else if (errno == ENOENT)
-			    oldw("can't provide Kerberos authentication data", "");
-			goto again;
+			    oldw("can't provide Kerberos authentication data");
+			execv(_PATH_RCP, argv_copy);
 		}
 	} else {
 		if (doencrypt)
 			errx(1,
 			   "the -x option requires Kerberos authentication");
 		if (geteuid() != 0) {
-		        fprintf(stderr, "rcp: not installed setuid root, only root may use non kerberized rcp\n");
-			exit(1);
+		    errx(1, "not installed setuid root, "
+			 "only root may use non kerberized rcp");
 		}
 		rem = rcmd(host, port, locuser, user, bp, 0);
 	}
@@ -502,29 +500,24 @@ toremote(char *targ, int argc, char **argv)
 			if (*src == 0)
 				src = ".";
 			host = strchr(argv[i], '@');
-			len = strlen(_PATH_RSH) + strlen(argv[i]) +
-			    strlen(src) + (tuser ? strlen(tuser) : 0) +
-			    strlen(thost) + strlen(targ) + CMDNEEDS + 20;
-			if (!(bp = malloc(len)))
-				err(1, " ");
 			if (host) {
-				*host++ = 0;
-				suser = argv[i];
-				if (*suser == '\0')
-					suser = pwd->pw_name;
-				else if (!okname(suser))
-					continue;
-				sprintf(bp,
-				    "%s %s -l %s -n %s %s '%s%s%s:%s'",
-				    _PATH_RSH, host, suser, cmd, src,
-				    tuser ? tuser : "", tuser ? "@" : "",
-				    thost, targ);
+			    *host++ = 0;
+			    suser = argv[i];
+			    if (*suser == '\0')
+				suser = pwd->pw_name;
+			    else if (!okname(suser))
+				continue;
+			    asprintf(&bp, "%s %s -l %s -n %s %s '%s%s%s:%s'",
+				     _PATH_RSH, host, suser, cmd, src,
+				     tuser ? tuser : "", tuser ? "@" : "",
+				     thost, targ);
 			} else
-				sprintf(bp,
-				    "exec %s %s -n %s %s '%s%s%s:%s'",
-				    _PATH_RSH, argv[i], cmd, src,
-				    tuser ? tuser : "", tuser ? "@" : "",
-				    thost, targ);
+			    asprintf(&bp, "exec %s %s -n %s %s '%s%s%s:%s'",
+				     _PATH_RSH, argv[i], cmd, src,
+				     tuser ? tuser : "", tuser ? "@" : "",
+				     thost, targ);
+			if(bp == NULL)
+			    errx(1, "out of memory");
 			susystem(bp, userid);
 			free(bp);
 		} else {			/* local to remote */
@@ -532,19 +525,28 @@ toremote(char *targ, int argc, char **argv)
 				len = strlen(targ) + CMDNEEDS + 20;
 				if (!(bp = malloc(len)))
 					err(1, " ");
-				sprintf(bp, "%s -t %s", cmd, targ);
+				snprintf(bp, len, "%s -t %s", cmd, targ);
 				host = thost;
 				if (use_kerberos)
 					rem = kerberos(&host, bp,
+#ifdef __CYGWIN32__
+						       tuser,
+#else
 					    pwd->pw_name,
+#endif
 					    tuser ? tuser : pwd->pw_name);
 				else
-					rem = rcmd(&host, port, pwd->pw_name,
+					rem = rcmd(&host, port,
+#ifdef __CYGWIN32__
+						   tuser,
+#else
+						   pwd->pw_name,
+#endif
 					    tuser ? tuser : pwd->pw_name,
 					    bp, 0);
 				if (rem < 0)
 					exit(1);
-#ifdef IP_TOS
+#if defined(IP_TOS) && defined(HAVE_SETSOCKOPT)
 				tos = IPTOS_THROUGHPUT;
 				if (setsockopt(rem, IPPROTO_IP, IP_TOS,
 				    (void *)&tos, sizeof(int)) < 0)
@@ -584,7 +586,7 @@ sink(int argc, char **argv)
 	if (!pflag)
 		umask(mask);
 	if (argc != 1) {
-		run_err("ambiguous target", "", "");
+		run_err("ambiguous target");
 		exit(1);
 	}
 	targ = *argv;
@@ -623,7 +625,13 @@ sink(int argc, char **argv)
 		if (ch == '\n')
 			*--cp = 0;
 
-#define getnum(t) (t) = 0; while (isdigit(*cp)) (t) = (t) * 10 + (*cp++ - '0');
+#define getnum(t)					\
+		do {					\
+		    (t) = 0;				\
+		    while (isdigit((unsigned char)*cp))	\
+			(t) = (t) * 10 + (*cp++ - '0');	\
+		} while(0)
+
 		cp = buf;
 		if (*cp == 'T') {
 			setimes++;
@@ -652,7 +660,7 @@ sink(int argc, char **argv)
 			 * followed.
 			 */
 			if (first) {
-				run_err("%s", cp, "");
+				run_err("%s", cp);
 				exit(1);
 			}
 			SCREWUP("expected control record");
@@ -666,7 +674,7 @@ sink(int argc, char **argv)
 		if (*cp++ != ' ')
 			SCREWUP("mode not delimited");
 
-		for (size = 0; isdigit(*cp);)
+		for (size = 0; isdigit((unsigned char)*cp);)
 			size = size * 10 + (*cp++ - '0');
 		if (*cp++ != ' ')
 			SCREWUP("size not delimited");
@@ -678,9 +686,9 @@ sink(int argc, char **argv)
 			need = strlen(targ) + strlen(cp) + 250;
 			if (need > cursize) {
 				if (!(namebuf = malloc(need)))
-					run_err("%s", strerror(errno), "");
+					run_err("%s", strerror(errno));
 			}
-			sprintf(namebuf, "%s%s%s", targ,
+			snprintf(namebuf, need, "%s%s%s", targ,
 			    *targ ? "/" : "", cp);
 			np = namebuf;
 		} else
@@ -710,7 +718,7 @@ sink(int argc, char **argv)
 				setimes = 0;
 				if (utime(np, &times) < 0)
 				    run_err("%s: set times: %s",
-					np, strerror(errno));
+					    np, strerror(errno));
 			}
 			if (mod_flag)
 				chmod(np, mode);
@@ -738,7 +746,7 @@ bad:			run_err("%s: %s", np, strerror(errno));
 				j = des_read(rem, cp, amt);
 				if (j <= 0) {
 					run_err("%s", j ? strerror(errno) :
-					    "dropped connection", "");
+						"dropped connection");
 					exit(1);
 				}
 				amt -= j;
@@ -768,14 +776,22 @@ bad:			run_err("%s: %s", np, strerror(errno));
 		}
 		if (pflag) {
 			if (exists || omode != mode)
+#ifdef HAVE_FCHMOD
 				if (fchmod(ofd, omode))
+#else
+				if (chmod(np, omode))
+#endif
 					run_err("%s: set mode: %s",
-					    np, strerror(errno));
+						np, strerror(errno));
 		} else {
 			if (!exists && omode != mode)
+#ifdef HAVE_FCHMOD
 				if (fchmod(ofd, omode & ~mask))
+#else
+				if (chmod(np, omode & ~mask))
+#endif
 					run_err("%s: set mode: %s",
-					    np, strerror(errno));
+						np, strerror(errno));
 		}
 		close(ofd);
 		response();
@@ -786,7 +802,7 @@ bad:			run_err("%s: %s", np, strerror(errno));
 			setimes = 0;
 			if (utime(np, &times) < 0) {
 				run_err("%s: set times: %s",
-				    np, strerror(errno));
+					np, strerror(errno));
 				wrerr = DISPLAYED;
 			}
 		}
@@ -802,7 +818,7 @@ bad:			run_err("%s: %s", np, strerror(errno));
 		}
 	}
 screwup:
-	run_err("protocol error: %s", why, "");
+	run_err("protocol error: %s", why);
 	exit(1);
 }
 
@@ -821,9 +837,9 @@ tolocal(int argc, char **argv)
 			    strlen(argv[argc - 1]) + 20;
 			if (!(bp = malloc(len)))
 				err(1, " ");
-			sprintf(bp, "exec %s%s%s %s %s", _PATH_CP,
-			    iamrecursive ? " -r" : "", pflag ? " -p" : "",
-			    argv[i], argv[argc - 1]);
+			snprintf(bp, len, "exec %s%s%s %s %s", _PATH_CP,
+				 iamrecursive ? " -r" : "", pflag ? " -p" : "",
+				 argv[i], argv[argc - 1]);
 			if (susystem(bp, userid))
 				++errs;
 			free(bp);
@@ -833,38 +849,58 @@ tolocal(int argc, char **argv)
 		if (*src == 0)
 			src = ".";
 		if ((host = strchr(argv[i], '@')) == NULL) {
+#ifdef __CYGWIN32__
+			errx (1, "Sorry, you need to specify the username");
+#else
 			host = argv[i];
 			suser = pwd->pw_name;
+#endif
 		} else {
 			*host++ = 0;
 			suser = argv[i];
 			if (*suser == '\0')
+#ifdef __CYGWIN32__
+			errx (1, "Sorry, you need to specify the username");
+#else
 				suser = pwd->pw_name;
+#endif
 			else if (!okname(suser))
 				continue;
 		}
 		len = strlen(src) + CMDNEEDS + 20;
 		if ((bp = malloc(len)) == NULL)
 			err(1, " ");
-		sprintf(bp, "%s -f %s", cmd, src);
+		snprintf(bp, len, "%s -f %s", cmd, src);
 		rem = 
 		    use_kerberos ? 
-			kerberos(&host, bp, pwd->pw_name, suser) : 
-			rcmd(&host, port, pwd->pw_name, suser, bp, 0);
+			kerberos(&host, bp,
+#ifndef __CYGWIN32__
+				 pwd->pw_name,
+#else
+				 suser,
+#endif
+				 suser) : 
+			rcmd(&host, port,
+#ifndef __CYGWIN32__
+			     pwd->pw_name,
+#else
+			     suser,
+#endif
+			     suser, bp, 0);
 		free(bp);
 		if (rem < 0) {
 			++errs;
 			continue;
 		}
 		seteuid(userid);
-#ifdef IP_TOS
+#if defined(IP_TOS) && defined(HAVE_SETSOCKOPT)
 		tos = IPTOS_THROUGHPUT;
 		if (setsockopt(rem, IPPROTO_IP, IP_TOS, (void *)&tos,
 			       sizeof(int)) < 0)
 			warn("TOS (ignored)");
 #endif /* IP_TOS */
 		if (doencrypt)
-		    send_auth(host, dest_realm);
+			send_auth(host, dest_realm);
 		sink(1, argv + argc - 1);
 		seteuid(0);
 		close(rem);
@@ -878,8 +914,28 @@ main(int argc, char **argv)
 {
 	int ch, fflag, tflag;
 	char *targ;
+	int i;
+
+	set_progname(argv[0]);
+
+	/*
+	 * Prepare for execing ourselves.
+	 */
+
+	argc_copy = argc + 1;
+	argv_copy = malloc((argc_copy + 1) * sizeof(*argv_copy));
+	if (argv_copy == NULL)
+	    err(1, "malloc");
+	argv_copy[0] = argv[0];
+	argv_copy[1] = "-K";
+	for(i = 1; i < argc; ++i) {
+	    argv_copy[i + 1] = strdup(argv[i]);
+	    if (argv_copy[i + 1] == NULL)
+		errx(1, "strdup: out of memory");
+	}
+	argv_copy[argc + 1] = NULL;
+
 
-	__progname = argv[0];
 	fflag = tflag = 0;
 	while ((ch = getopt(argc, argv, OPTIONS)) != EOF)
 		switch(ch) {			/* User-visible flags. */
@@ -888,7 +944,7 @@ main(int argc, char **argv)
 			break;
 		case 'k':
 			dest_realm = dst_realm_buf;
-			strncpy(dst_realm_buf, optarg, REALM_SZ);
+			strcpy_truncate(dst_realm_buf, optarg, REALM_SZ);
 			break;
 		case 'x':
 			doencrypt = 1;
@@ -919,40 +975,44 @@ main(int argc, char **argv)
 	argc -= optind;
 	argv += optind;
 
-	port=get_shell_port(use_kerberos, doencrypt);
+	/* Rcp implements encrypted file transfer without using the
+	 * kshell service, pass 0 for no encryption */
+	port = get_shell_port(use_kerberos, 0);
 
-	if ((pwd = k_getpwuid(userid = getuid())) == NULL)
+	userid = getuid();
+
+#ifndef __CYGWIN32__
+	if ((pwd = k_getpwuid(userid)) == NULL)
 		errx(1, "unknown user %d", (int)userid);
+#endif
 
 	rem = STDIN_FILENO;		/* XXX */
 
-	if (fflag) {			/* Follow "protocol", send data. */
-		if (doencrypt)
-		    answer_auth();
-		response();
-		setuid(userid);
-		if (k_hasafs()) {
-		  char cell[64];
-		  if (k_afs_cell_of_file(pwd->pw_dir, cell, sizeof(cell)) == 0)
-		    k_afsklog(cell, 0);
-		  k_afsklog(0, 0);
-		}
-		source(argc, argv);
-		exit(errs);
-	}
-
-	if (tflag) {			/* Receive data. */
+	if (fflag || tflag) {
 	    if (doencrypt)
-		    answer_auth();
-		setuid(userid);
-		if (k_hasafs()) {
-		  char cell[64];
-		  if (k_afs_cell_of_file(pwd->pw_dir, cell, sizeof(cell)) == 0)
-		    k_afsklog(cell, 0);
-		  k_afsklog(0, 0);
-		}
+		answer_auth();
+	    if(fflag)
+		response();
+	    if(do_osfc2_magic(pwd->pw_uid))
+		exit(1);
+	    setuid(userid);
+	    if (k_hasafs()) {
+		/* Sometimes we will need cell specific tokens
+		 * to be able to read and write files, thus,
+		 * the token stuff done in rshd might not
+		 * suffice.
+		 */
+		char cell[64];
+		if (k_afs_cell_of_file(pwd->pw_dir,
+				       cell, sizeof(cell)) == 0)
+		    krb_afslog(cell, 0);
+		krb_afslog(0, 0);
+	    }
+	    if(fflag)
+		source(argc, argv);
+	    else
 		sink(argc, argv);
-		exit(errs);
+	    exit(errs);
 	}
 
 	if (argc < 2)
@@ -962,10 +1022,10 @@ main(int argc, char **argv)
 
 	rem = -1;
 	/* Command to be executed on remote system using "rsh". */
-	sprintf(cmd,
-	    "rcp%s%s%s%s", iamrecursive ? " -r" : "",
-	    (doencrypt && use_kerberos ? " -x" : ""),
-	    pflag ? " -p" : "", targetshouldbedirectory ? " -d" : "");
+	snprintf(cmd, sizeof(cmd),
+		 "rcp%s%s%s%s", iamrecursive ? " -r" : "",
+		 (doencrypt && use_kerberos ? " -x" : ""),
+		 pflag ? " -p" : "", targetshouldbedirectory ? " -d" : "");
 
 	signal(SIGPIPE, lostconn);
 
