@@ -36,7 +36,7 @@
  */
 
 #include "ftp_locl.h"
-RCSID("$Id: cmds.c,v 1.30 1997/11/07 19:51:54 joda Exp $");
+RCSID("$Id: cmds.c,v 1.34.2.1 1999/08/18 18:19:44 assar Exp $");
 
 typedef void (*sighand)(int);
 
@@ -119,12 +119,17 @@ setpeer(int argc, char **argv)
 		/*
 		 * Set up defaults for FTP.
 		 */
-		strcpy(typename, "ascii"), type = TYPE_A;
+		strcpy_truncate(typename, "ascii", sizeof(typename));
+		type = TYPE_A;
 		curtype = TYPE_A;
-		strcpy(formname, "non-print"), form = FORM_N;
-		strcpy(modename, "stream"), mode = MODE_S;
-		strcpy(structname, "file"), stru = STRU_F;
-		strcpy(bytename, "8"), bytesize = 8;
+		strcpy_truncate(formname, "non-print", sizeof(formname));
+		form = FORM_N;
+		strcpy_truncate(modename, "stream", sizeof(modename));
+		mode = MODE_S;
+		strcpy_truncate(structname, "file", sizeof(structname));
+		stru = STRU_F;
+		strcpy_truncate(bytename, "8", sizeof(bytename));
+		bytesize = 8;
 		if (autologin)
 			login(argv[1]);
 
@@ -165,7 +170,7 @@ setpeer(int argc, char **argv)
 			 * for text files unless changed by the user.
 			 */
 			type = 0;
-			strcpy(typename, "binary");
+			strcpy_truncate(typename, "binary", sizeof(typename));
 			if (overbose)
 			    printf("Using %s mode to transfer files.\n",
 				typename);
@@ -238,7 +243,7 @@ settype(int argc, char **argv)
 	else
 		comret = command("TYPE %s", p->t_mode);
 	if (comret == COMPLETE) {
-		strcpy(typename, p->t_name);
+		strcpy_truncate(typename, p->t_name, sizeof(typename));
 		curtype = type = p->t_type;
 	}
 }
@@ -398,7 +403,8 @@ usage:
 		argv[2] = domap(argv[2]);
 	}
 	sendrequest(cmd, argv[1], argv[2],
-	    argv[1] != oldargv1 || argv[2] != oldargv2);
+		    curtype == TYPE_I ? "rb" : "r",
+		    argv[1] != oldargv1 || argv[2] != oldargv2);
 }
 
 /* ARGSUSED */
@@ -476,7 +482,9 @@ mput(int argc, char **argv)
 		    tp = domap(tp);
 		}
 		sendrequest((sunique) ? "STOU" : "STOR",
-			    cp, tp, cp != tp || !interactive);
+			    cp, tp,
+			    curtype == TYPE_I ? "rb" : "r",
+			    cp != tp || !interactive);
 		if (!mflag && fromatty) {
 		    ointer = interactive;
 		    interactive = 1;
@@ -501,7 +509,9 @@ mput(int argc, char **argv)
 		tp = (ntflag) ? dotrans(argv[i]) : argv[i];
 		tp = (mapflag) ? domap(tp) : tp;
 		sendrequest((sunique) ? "STOU" : "STOR",
-			    argv[i], tp, tp != argv[i] || !interactive);
+			    argv[i],
+			    curtype == TYPE_I ? "rb" : "r",
+			    tp, tp != argv[i] || !interactive);
 		if (!mflag && fromatty) {
 		    ointer = interactive;
 		    interactive = 1;
@@ -526,7 +536,9 @@ mput(int argc, char **argv)
 		tp = (ntflag) ? dotrans(*cpp) : *cpp;
 		tp = (mapflag) ? domap(tp) : tp;
 		sendrequest((sunique) ? "STOU" : "STOR",
-			    *cpp, tp, *cpp != tp || !interactive);
+			    *cpp, tp,
+			    curtype == TYPE_I ? "rb" : "r",
+			    *cpp != tp || !interactive);
 		if (!mflag && fromatty) {
 		    ointer = interactive;
 		    interactive = 1;
@@ -546,13 +558,26 @@ mput(int argc, char **argv)
 void
 reget(int argc, char **argv)
 {
-    getit(argc, argv, 1, "r+w");
+    getit(argc, argv, 1, curtype == TYPE_I ? "r+wb" : "r+w");
 }
 
 void
 get(int argc, char **argv)
 {
-    getit(argc, argv, 0, restart_point ? "r+w" : "w" );
+    char *mode;
+
+    if (restart_point)
+	if (curtype == TYPE_I)
+	    mode = "r+wb";
+	else
+	    mode = "r+w";
+    else
+	if (curtype == TYPE_I)
+	    mode = "wb";
+	else
+	    mode = "w";
+
+    getit(argc, argv, 0, mode);
 }
 
 /*
@@ -617,41 +642,45 @@ getit(int argc, char **argv, int restartit, char *mode)
 				return (0);
 			}
 			restart_point = stbuf.st_size;
-		} else {
-			if (ret == 0) {
-				int overbose;
+		} else if (ret == 0) {
+			int overbose;
+			int cmdret;
+			int yy, mo, day, hour, min, sec;
+			struct tm *tm;
 
-				overbose = verbose;
-				if (debug == 0)
-					verbose = -1;
-				if (command("MDTM %s", argv[1]) == COMPLETE) {
-					int yy, mo, day, hour, min, sec;
-					struct tm *tm;
-					verbose = overbose;
-					sscanf(reply_string,
-					    "%*s %04d%02d%02d%02d%02d%02d",
-					    &yy, &mo, &day, &hour, &min, &sec);
-					tm = gmtime(&stbuf.st_mtime);
-					tm->tm_mon++;
-					if (tm->tm_year > yy%100)
-						return (1);
-					if ((tm->tm_year == yy%100 && 
-					    tm->tm_mon > mo) ||
-					   (tm->tm_mon == mo && 
-					    tm->tm_mday > day) ||
-					   (tm->tm_mday == day && 
-					    tm->tm_hour > hour) ||
-					   (tm->tm_hour == hour && 
-					    tm->tm_min > min) ||
-					   (tm->tm_min == min && 
-					    tm->tm_sec > sec))
-						return (1);
-				} else {
-					printf("%s\n", reply_string);
-					verbose = overbose;
-					return (0);
-				}
+			overbose = verbose;
+			if (debug == 0)
+				verbose = -1;
+			cmdret = command("MDTM %s", argv[1]);
+			verbose = overbose;
+			if (cmdret != COMPLETE) {
+				printf("%s\n", reply_string);
+				return (0);
 			}
+			if (sscanf(reply_string,
+				   "%*s %04d%02d%02d%02d%02d%02d",
+				   &yy, &mo, &day, &hour, &min, &sec)
+			    != 6) {
+				printf ("bad MDTM result\n");
+				return (0);
+			}
+
+			tm = gmtime(&stbuf.st_mtime);
+			tm->tm_mon++;
+			tm->tm_year += 1900;
+
+			if ((tm->tm_year > yy) ||
+			    (tm->tm_year == yy && 
+			     tm->tm_mon > mo) ||
+			    (tm->tm_mon == mo && 
+			     tm->tm_mday > day) ||
+			    (tm->tm_mday == day && 
+			     tm->tm_hour > hour) ||
+			    (tm->tm_hour == hour && 
+			     tm->tm_min > min) ||
+			    (tm->tm_min == min && 
+			     tm->tm_sec > sec))
+				return (1);
 		}
 	}
 
@@ -707,8 +736,9 @@ mget(int argc, char **argv)
 			if (mapflag) {
 				tp = domap(tp);
 			}
-			recvrequest("RETR", tp, cp, "w",
-			    tp != cp || !interactive, 0);
+			recvrequest("RETR", tp, cp,
+				    curtype == TYPE_I ? "wb" : "w",
+				    tp != cp || !interactive, 0);
 			if (!mflag && fromatty) {
 				ointer = interactive;
 				interactive = 1;
@@ -754,7 +784,7 @@ remglob(char **argv, int doswitch)
     }
     if (ftemp == NULL) {
 	int fd;
-	strcpy(temp, _PATH_TMP_XXX);
+	strcpy_truncate(temp, _PATH_TMP_XXX, sizeof(temp));
 	fd = mkstemp(temp);
 	if(fd < 0){
 	    warn("unable to create temporary file %s", temp);
@@ -1210,8 +1240,8 @@ shell(int argc, char **argv)
 		namep = strrchr(shell,'/');
 		if (namep == NULL)
 			namep = shell;
-		strcpy(shellnam,"-");
-		strcat(shellnam, ++namep);
+		snprintf (shellnam, sizeof(shellnam),
+			  "-%s", ++namep);
 		if (strcmp(namep, "sh") != 0)
 			shellnam[0] = '+';
 		if (debug) {
@@ -1386,22 +1416,19 @@ site(int argc, char **argv)
 void
 quote1(char *initial, int argc, char **argv)
 {
-	int i, len;
-	char buf[BUFSIZ];		/* must be >= sizeof(line) */
+    int i;
+    char buf[BUFSIZ];		/* must be >= sizeof(line) */
 
-	strcpy(buf, initial);
-	if (argc > 1) {
-		len = strlen(buf);
-		len += strlen(strcpy(&buf[len], argv[1]));
-		for (i = 2; i < argc; i++) {
-			buf[len++] = ' ';
-			len += strlen(strcpy(&buf[len], argv[i]));
-		}
-	}
-	if (command(buf) == PRELIM) {
-		while (getreply(0) == PRELIM)
-			continue;
-	}
+    strcpy_truncate(buf, initial, sizeof(buf));
+    for(i = 1; i < argc; i++) {
+	if(i > 1)
+	    strcat_truncate(buf, " ", sizeof(buf));
+	strcat_truncate(buf, argv[i], sizeof(buf));
+    }
+    if (command("%s", buf) == PRELIM) {
+	while (getreply(0) == PRELIM)
+	    continue;
+    }
 }
 
 void
@@ -1484,7 +1511,7 @@ disconnect(int argc, char **argv)
 	}
 	cout = NULL;
 	connected = 0;
-	krb4_quit();
+	sec_end();
 	data = -1;
 	if (!proxy) {
 		macnum = 0;
@@ -1548,12 +1575,11 @@ account(int argc, char **argv)
 	if (argc > 1) {
 		++argv;
 		--argc;
-		strncpy(acct,*argv,49);
-		acct[49] = '\0';
+		strcpy_truncate (acct, *argv, sizeof(acct));
 		while (argc > 1) {
 			--argc;
 			++argv;
-			strncat(acct,*argv, 49-strlen(acct));
+			strcat_truncate(acct, *argv, sizeof(acct));
 		}
 	}
 	else {
@@ -1665,14 +1691,12 @@ setntrans(int argc, char **argv)
 	}
 	ntflag++;
 	code = ntflag;
-	strncpy(ntin, argv[1], 16);
-	ntin[16] = '\0';
+	strcpy_truncate (ntin, argv[1], 17);
 	if (argc == 2) {
 		ntout[0] = '\0';
 		return;
 	}
-	strncpy(ntout, argv[2], 16);
-	ntout[16] = '\0';
+	strcpy_truncate (ntout, argv[2], 17);
 }
 
 char *
@@ -1729,10 +1753,10 @@ setnmap(int argc, char **argv)
 		cp = strchr(altarg, ' ');
 	}
 	*cp = '\0';
-	strncpy(mapin, altarg, MaxPathLen - 1);
+	strcpy_truncate(mapin, altarg, MaxPathLen);
 	while (*++cp == ' ')
 		continue;
-	strncpy(mapout, cp, MaxPathLen - 1);
+	strcpy_truncate(mapout, cp, MaxPathLen);
 }
 
 char *
@@ -1984,7 +2008,9 @@ macdef(int argc, char **argv)
 	if (interactive) {
 		printf("Enter macro line by line, terminating it with a null line\n");
 	}
-	strncpy(macros[macnum].mac_name, argv[1], 8);
+	strcpy_truncate(macros[macnum].mac_name,
+			argv[1],
+			sizeof(macros[macnum].mac_name));
 	if (macnum == 0) {
 		macros[macnum].mac_start = macbuf;
 	}
@@ -2084,7 +2110,7 @@ void
 newer(int argc, char **argv)
 {
 
-	if (getit(argc, argv, -1, "w"))
+	if (getit(argc, argv, -1, curtype == TYPE_I ? "wb" : "w"))
 		printf("Local file \"%s\" is newer than remote file \"%s\"\n",
 			argv[2], argv[1]);
 }
