@@ -38,7 +38,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: logwtmp.c,v 1.12 1998/01/09 20:30:20 assar Exp $");
+RCSID("$Id: logwtmp.c,v 1.13 1999/03/01 09:49:37 joda Exp $");
 #endif
 
 #include <stdio.h>
@@ -87,7 +87,7 @@ ftpd_logwtmp(char *line, char *name, char *host)
 #endif
 
     memset(&ut, 0, sizeof(struct utmp));
-#ifdef HAVE_UT_TYPE
+#ifdef HAVE_STRUCT_UTMP_UT_TYPE
     if(name[0])
 	ut.ut_type = USER_PROCESS;
     else
@@ -95,10 +95,10 @@ ftpd_logwtmp(char *line, char *name, char *host)
 #endif
     strncpy(ut.ut_line, line, sizeof(ut.ut_line));
     strncpy(ut.ut_name, name, sizeof(ut.ut_name));
-#ifdef HAVE_UT_PID
+#ifdef HAVE_STRUCT_UTMP_UT_PID
     ut.ut_pid = getpid();
 #endif
-#ifdef HAVE_UT_HOST
+#ifdef HAVE_STRUCT_UTMP_UT_HOST
     strncpy(ut.ut_host, host, sizeof(ut.ut_host));
 #endif
     ut.ut_time = time(NULL);
@@ -107,7 +107,7 @@ ftpd_logwtmp(char *line, char *name, char *host)
     strncpy(utx.ut_line, line, sizeof(utx.ut_line));
     strncpy(utx.ut_user, name, sizeof(utx.ut_user));
     strncpy(utx.ut_host, host, sizeof(utx.ut_host));
-#ifdef HAVE_UT_SYSLEN
+#ifdef HAVE_STRUCT_UTMPX_UT_SYSLEN
     utx.ut_syslen = strlen(host) + 1;
     if (utx.ut_syslen > sizeof(utx.ut_host))
         utx.ut_syslen = sizeof(utx.ut_host);
