@@ -18,7 +18,7 @@
 
 #include "sample.h"
 
-RCSID("$Id: sample_server.c,v 1.10 1997/05/02 14:28:20 assar Exp $");
+RCSID("$Id: sample_server.c,v 1.12 1999/03/10 18:33:53 joda Exp $");
 
 static void
 usage (void)
@@ -51,18 +51,18 @@ main(int argc, char **argv)
 
     set_progname (argv[0]);
 
-    openlog(__progname, LOG_ODELAY, LOG_DAEMON);
+    roken_openlog(__progname, LOG_ODELAY, LOG_DAEMON);
 
-    strcpy (service, SAMPLE_SERVICE);
+    strcpy_truncate (service, SAMPLE_SERVICE, sizeof(service));
     *srvtab = '\0';
 
     while ((c = getopt (argc, argv, "s:t:i")) != EOF)
 	switch (c) {
 	case 's' :
-	    strncpy (service, optarg, sizeof(service));
+	    strcpy_truncate (service, optarg, sizeof(service));
 	    break;
 	case 't' :
-	    strncpy (srvtab, optarg, sizeof(srvtab));
+	    strcpy_truncate (srvtab, optarg, sizeof(srvtab));
 	    break;
 	case 'i':
 	    no_inetd = 1;
@@ -122,8 +122,9 @@ main(int argc, char **argv)
 	/* now that we have decoded the authenticator, translate
 	   the kerberos principal.instance@realm into a local name */
 	if (krb_kntoln(&auth_data, lname) != KSUCCESS)
-	    strcpy(lname,
-		   "*No local name returned by krb_kntoln*");
+	    strcpy_truncate(lname,
+			    "*No local name returned by krb_kntoln*",
+			    sizeof(lname));
 	/* compose the reply */
 	snprintf(retbuf, sizeof(retbuf),
 		"You are %s.%s@%s (local name %s),\n at address %s, version %s, cksum %ld\n",
