@@ -33,7 +33,7 @@
 
 #include "telnet_locl.h"
 
-RCSID("$Id: commands.c,v 1.45 1998/05/01 05:16:55 assar Exp $");
+RCSID("$Id: commands.c,v 1.53 1999/07/07 14:56:17 assar Exp $");
 
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 int tos = -1;
@@ -66,7 +66,8 @@ makeargv()
     margc = 0;
     cp = line;
     if (*cp == '!') {		/* Special case shell escape */
-	strcpy(saveline, line);	/* save for shell command */
+	/* save for shell command */
+	strcpy_truncate(saveline, line, sizeof(saveline));
 	*argp++ = "!";		/* No room in string to get this */
 	margc++;
 	cp++;
@@ -451,6 +452,19 @@ togdebug()
     return 1;
 }
 
+#if defined(KRB4) && defined(HAVE_KRB_DISABLE_DEBUG)
+#include <krb.h>
+
+static int
+togkrbdebug(void)
+{
+    if(krb_debug)
+	krb_enable_debug();
+    else
+	krb_disable_debug();
+    return 1;
+}
+#endif
 
 static int
 togcrlf()
@@ -661,6 +675,13 @@ static struct togglelist Togglelist[] = {
 	    togdebug,
 		&debug,
 		    "turn on socket level debugging" },
+#if defined(KRB4) && defined(HAVE_KRB_DISABLE_DEBUG)
+    { "krb_debug",
+      "kerberos 4 debugging",
+      togkrbdebug,
+      &krb_debug,
+      "turn on kerberos 4 debugging" },
+#endif
     { "netdata",
 	"printing of hexadecimal network data (debugging)",
 	    0,
@@ -1561,9 +1582,8 @@ env_init(void)
 		/* If this is not the full name, try to get it via DNS */
 		if (strchr(hbuf, '.') == 0) {
 			struct hostent *he = roken_gethostbyname(hbuf);
-			if (he != 0)
-				strncpy(hbuf, he->h_name, 256);
-			hbuf[256] = '\0';
+			if (he != NULL)
+				strcpy_truncate(hbuf, he->h_name, 256);
 		}
 
 		asprintf (&cp, "%s%s", hbuf, cp2);
@@ -1939,7 +1959,7 @@ status(int argc, char **argv)
  * Function that gets called when SIGINFO is received.
  */
 void
-ayt_status(void)
+ayt_status(int ignore)
 {
     call(status, "status", "notmuch", 0);
 }
@@ -1961,7 +1981,7 @@ cmdrc(char *m1, char *m2)
     if (skiprc)
 	return;
 
-    strcpy(m1save, m1);
+    strcpy_truncate(m1save, m1, sizeof(m1save));
     m1 = m1save;
 
     if (rcname[0] == 0) {
@@ -2033,8 +2053,8 @@ tn(int argc, char **argv)
     struct sockaddr_in6 sin6;
 #endif
     struct sockaddr_in sin;
-    struct sockaddr *sa;
-    int sa_size;
+    struct sockaddr *sa = NULL;
+    int sa_size = 0;
     struct servent *sp = 0;
     unsigned long temp;
     extern char *inet_ntoa();
@@ -2042,8 +2062,10 @@ tn(int argc, char **argv)
     char *srp = 0;
     int srlen;
 #endif
-    char *cmd, *hostp = 0, *portp = 0, *user = 0;
-    int family, port;
+    char *cmd, *hostp = 0, *portp = 0;
+    char *user = 0;
+    int family, port = 0;
+    char **addr_list;
 
     /* clear the socket address prior to use */
 
@@ -2053,7 +2075,7 @@ tn(int argc, char **argv)
 	return 0;
     }
     if (argc < 2) {
-	strcpy(line, "open ");
+	strcpy_truncate(line, "open ", sizeof(line));
 	printf("(to) ");
 	fgets(&line[strlen(line)], sizeof(line) - strlen(line), stdin);
 	makeargv();
@@ -2124,7 +2146,7 @@ tn(int argc, char **argv)
 	    sin6.sin6_family = family = AF_INET6;
 	    sa = (struct sockaddr *)&sin6;
 	    sa_size = sizeof(sin6);
-	    strcpy(_hostname, hostp);
+	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
 	    hostname =_hostname;
 	} else
 #endif
@@ -2132,19 +2154,22 @@ tn(int argc, char **argv)
 	    sin.sin_family = family = AF_INET;
 	    sa = (struct sockaddr *)&sin;
 	    sa_size = sizeof(sin);
-	    strcpy(_hostname, hostp);
+	    strcpy_truncate(_hostname, hostp, sizeof(_hostname));
 	    hostname = _hostname;
 	} else {
 #ifdef HAVE_GETHOSTBYNAME2
+#ifdef HAVE_IPV6
 	    host = gethostbyname2(hostp, AF_INET6);
 	    if(host == NULL)
+#endif
 		host = gethostbyname2(hostp, AF_INET);
 #else
 	    host = roken_gethostbyname(hostp);
 #endif
 	    if (host) {
-		strncpy(_hostname, host->h_name, sizeof(_hostname));
+		strcpy_truncate(_hostname, host->h_name, sizeof(_hostname));
 		family = host->h_addrtype;
+		addr_list = host->h_addr_list;
 
 		switch(family) {
 		case AF_INET:
@@ -2152,7 +2177,7 @@ tn(int argc, char **argv)
 		    sa_size = sizeof(sin);
 		    sa = (struct sockaddr *)&sin;
 		    sin.sin_family = family;
-		    sin.sin_addr   = *((struct in_addr *)(*host->h_addr_list));
+		    sin.sin_addr   = *((struct in_addr *)(*addr_list));
 		    break;
 #ifdef HAVE_IPV6
 		case AF_INET6:
@@ -2160,7 +2185,7 @@ tn(int argc, char **argv)
 		    sa_size = sizeof(sin6);
 		    sa = (struct sockaddr *)&sin6;
 		    sin6.sin6_family = family;
-		    sin6.sin6_addr   = *((struct in6_addr *)(*host->h_addr_list));
+		    sin6.sin6_addr   = *((struct in6_addr *)(*addr_list));
 		    break;
 #endif
 		default:
@@ -2253,7 +2278,7 @@ tn(int argc, char **argv)
 #endif
 #if	defined(IPPROTO_IP) && defined(IP_TOS)
 	{
-# if	defined(HAS_GETTOS)
+# if	defined(HAVE_GETTOSBYNAME)
 	    struct tosent *tp;
 	    if (tos < 0 && (tp = gettosbyname("telnet", "tcp")))
 		tos = tp->t_tos;
@@ -2273,14 +2298,14 @@ tn(int argc, char **argv)
 	}
 
 	if (connect(net, sa, sa_size) < 0) {
-	    if (host && host->h_addr_list[1]) {
+	    if (host && addr_list[1]) {
 		int oerrno = errno;
 
 		switch(family) {
 		case AF_INET :
 		    fprintf(stderr, "telnet: connect to address %s: ",
 			    inet_ntoa(sin.sin_addr));
-		    sin.sin_addr = *((struct in_addr *)(*++host->h_addr_list));
+		    sin.sin_addr = *((struct in_addr *)(*++addr_list));
 		    break;
 #ifdef HAVE_IPV6
 		case AF_INET6: {
@@ -2289,7 +2314,7 @@ tn(int argc, char **argv)
 		    fprintf(stderr, "telnet: connect to address %s: ",
 			    inet_ntop(AF_INET6, &sin6.sin6_addr, buf,
 				      sizeof(buf)));
-		    sin6.sin6_addr = *((struct in6_addr *)(*++host->h_addr_list));
+		    sin6.sin6_addr = *((struct in6_addr *)(*++addr_list));
 		    break;
 		}
 #endif
@@ -2311,32 +2336,22 @@ tn(int argc, char **argv)
 #endif
     } while (connected == 0);
     cmdrc(hostp, hostname);
-    if (autologin && user == NULL) {
-	struct passwd *pw;
-
-	user = getenv("USER");
-	if (user == NULL ||
-	    ((pw = k_getpwnam(user)) && pw->pw_uid != getuid())) {
-		if ((pw = k_getpwuid(getuid())))
-			user = pw->pw_name;
-		else
-			user = NULL;
-	}
-    }
+    if (autologin && user == NULL)
+	user = (char *)get_default_username ();
     if (user) {
 	env_define((unsigned char *)"USER", (unsigned char *)user);
 	env_export((unsigned char *)"USER");
     }
     call(status, "status", "notmuch", 0);
     if (setjmp(peerdied) == 0)
-	telnet(user);
+	my_telnet((char *)user);
     NetClose(net);
     ExitString("Connection closed by foreign host.\r\n",1);
     /*NOTREACHED*/
     return 0;
 }
 
-#define HELPINDENT (sizeof ("connect"))
+#define HELPINDENT ((int)sizeof ("connect"))
 
 static char
 	openhelp[] =	"connect to a site",
@@ -2647,14 +2662,9 @@ sourceroute(char *arg, char **cpp, int *lenp)
 		if ((tmp = inet_addr(cp)) != -1) {
 			sin_addr.s_addr = tmp;
 		} else if ((host = roken_gethostbyname(cp))) {
-#if	defined(h_addr)
 			memmove(&sin_addr,
 				host->h_addr_list[0],
 				sizeof(sin_addr));
-#else
-			memmove(&sin_addr, host->h_addr,
-				sizeof(sin_addr));
-#endif
 		} else {
 			*cpp = cp;
 			return(0);
