@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -43,7 +43,7 @@
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-RCSID("$Id: passwd_dlg.c,v 1.5 1997/04/20 06:35:15 assar Exp $");
+RCSID("$Id: passwd_dlg.c,v 1.6 1998/06/09 19:25:09 joda Exp $");
 #endif
 
 #ifdef WIN32	/* Visual C++ 4.0 (Windows95/NT) */
@@ -84,12 +84,12 @@ pwd_dialog(char *buf, int size)
     switch(DialogBox(hInst,MAKEINTRESOURCE(IDD_PASSWD_DIALOG),wnd,pwd_dialog_proc))
     {
     case IDOK:
-	strcpy(buf,passwd);
-	for(i=0; passwd[i] != '\0'; i++) passwd[i] = '\0';
+	strcpy_truncate(buf, passwd, size);
+	memset (passwd, 0, sizeof(passwd));
 	return 0;
     case IDCANCEL:
     default:
-	for(i=0; passwd[i] != '\0'; i++) passwd[i] = '\0';
+	memset (passwd, 0, sizeof(passwd));
 	return 1;
     }
 }
