@@ -38,7 +38,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_priv.c,v 1.20 1998/02/19 19:13:40 joda Exp $");
+RCSID("$Id: mk_priv.c,v 1.21 1998/06/09 19:25:23 joda Exp $");
 
 /* application include files */
 #include "krb-archaeology.h"
@@ -93,16 +93,16 @@ krb_mk_priv(void *in, void *out, u_int32_t length,
     u_int32_t src_addr;
     u_int32_t len;
 
-    p += krb_put_int(KRB_PROT_VERSION, p, 1);
-    p += krb_put_int(AUTH_MSG_PRIVATE, p, 1);
+    p += krb_put_int(KRB_PROT_VERSION, p, 1, 1);
+    p += krb_put_int(AUTH_MSG_PRIVATE, p, 1, 1);
 
     len = 4 + length + 1 + 4 + 4;
     len = (len + 7) & ~7;
-    p += krb_put_int(len, p, 4);
+    p += krb_put_int(len, p, 4, 4);
     
     cipher = p;
 
-    p += krb_put_int(length, p, 4);
+    p += krb_put_int(length, p, 4, 4);
     
     memcpy(p, in, length);
     p += length;
@@ -112,9 +112,9 @@ krb_mk_priv(void *in, void *out, u_int32_t length,
     *p++ =tv.tv_usec / 5000;
     
     src_addr = sender->sin_addr.s_addr;
-    p += krb_put_address(src_addr, p);
+    p += krb_put_address(src_addr, p, 4);
 
-    p += krb_put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4);
+    p += krb_put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4, 4);
     
     memset(p, 0, 7);
 
